/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-auth.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-auth.h"
#include "sussen-gconf.h"
#include "sussen-util.h"

#include "nessus.h"
#include "globals.h"

GladeXML *xml;
GtkWidget *login_button;

gchar *g_host;
gchar *g_port;

void do_server_login (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	gint port = (gint) atoi(g_port);
	const gchar *username, *password;
	gchar *err;

	w = glade_xml_get_widget (xml,"auth_username_entry");
	username = gtk_entry_get_text (GTK_ENTRY(w));
	
	w = glade_xml_get_widget (xml,"auth_password_entry");
	password = gtk_entry_get_text (GTK_ENTRY(w));
		
	err = connect_to_nessusd (g_host,port,username,password);
	if (err) {
		update_status (err);
	} else {
		update_status ("Successfully connected");
	}
}

void set_login_button_state (GtkWidget *widget, gpointer user_data) {

	const gchar *widget_text;
	const gchar *user_data_text;
	
	gint ud_text_len = 0;
	
	user_data_text = gtk_entry_get_text (GTK_ENTRY(user_data));
	if (user_data_text) {
		ud_text_len = strlen (user_data_text);
	}	

	widget_text = gtk_entry_get_text (GTK_ENTRY(widget));
	if (widget_text) {
		if (strlen(widget_text) > 0 && ud_text_len > 0) {
			gtk_widget_set_sensitive (login_button, TRUE);
		} else { 
			gtk_widget_set_sensitive (login_button, FALSE);
		}
	}
}

void login_nessusd (gchar *nessusd_host, gchar *nessusd_port, gchar *nessusd_user) {
	
	GtkWidget *auth_dialog;
	GtkWidget *username_entry;
	GtkWidget *w;
	
	g_host = g_strdup (nessusd_host);
	g_port = g_strdup (nessusd_port);
	
	/* 
	 * Create auth dialog dialog
	 */

	xml = glade_xml_new(PACKAGE_DATA_DIR"/auth.glade", NULL, NULL);	
	auth_dialog = glade_xml_get_widget (xml,"sussen_auth_dialog");

	/*
	 * Set username
	 */
	
	username_entry = glade_xml_get_widget (xml, "auth_username_entry");
	if (nessusd_user) {
		gtk_entry_set_text (GTK_ENTRY(username_entry), nessusd_user);
		
		w = glade_xml_get_widget (xml, "auth_password_entry");
		gtk_widget_grab_focus (w);
	} else {
		gtk_widget_grab_focus (username_entry);
	}
	
	/* HIG: Authentication Alerts
	 * ref: http://developer.gnome.org/projects/gup/hig/1.0/windows.html#alerts-authentication
	 *  
	 * - does not enable the OK or equivalent button until all fields have been filled 
	 *   by the user
	 */

	w = glade_xml_get_widget (xml, "auth_password_entry");
	g_signal_connect (GTK_OBJECT (username_entry),
					  "changed",
					  G_CALLBACK (set_login_button_state),
					  (gpointer) w);
	
		
	g_signal_connect (GTK_OBJECT (w),
					  "changed",
					  G_CALLBACK (set_login_button_state),
					  (gpointer) username_entry);
	
	login_button = glade_xml_get_widget (xml,"auth_login_button");
	gtk_widget_set_sensitive (login_button, FALSE);	
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	w = login_button;	
	g_signal_connect (GTK_OBJECT (w),
					  "clicked",
					  G_CALLBACK (do_server_login),
					  NULL);
	
	
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) auth_dialog);
	
	w = glade_xml_get_widget (xml,"auth_cancel_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) auth_dialog);
}

void logout_nessusd (void) {
	
	shutdown (GlobalSocket,2);
	close (GlobalSocket);
	GlobalSocket = -1;
	
	update_status ("Logged out");
}
