/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-fileio.h
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-fileio.h"
#include "sussen-preferences.h"
#include <glib.h>
#include <pwd.h>
#include <sys/types.h>

GIOChannel *channel;

void list_callback (gpointer data, gpointer user_data) {
	
	gchar *buf = g_malloc (2000);
	gchar *tmp;
	gchar *host;
	gchar *port;
	gchar *user;
	gchar *enabled;
	gsize *bytes_written = 0;
	gint status;
			
	tmp = g_strdup ((gchar*)data);
	host = strtok (tmp, ":");  
	port = strtok (NULL, ":");
	user = strtok (NULL, ":");
	enabled = strtok (NULL, ":");
			
	g_snprintf (buf,2000,"<server name=\"%s\" port=\"%s\" user=\"%s\" enabled=\"%s\" />\n",host,port,user,enabled);
	status = g_io_channel_write_chars (channel,buf,-1,bytes_written,NULL);
			
	if (status != G_IO_STATUS_NORMAL) {
		switch (status) {
	
			case G_IO_STATUS_ERROR:
				g_print ("G_IO_STATUS_ERROR\n");
				break;
				
			case G_IO_STATUS_EOF:
				g_print ("G_IO_STATUS_EOF\n");
				break;
			
			case G_IO_STATUS_AGAIN:
				g_print ("G_IO_STATUS_AGAIN\n");
				break;
		}	
	}	
	g_free (buf);
}

gchar* get_server_xml_filename (void) {

	gchar *home;
	gchar *ret;
	struct passwd *pwd;
  
	home = getenv("HOME");
	if(home) {
		ret = g_malloc(strlen(home)+strlen("/.sussen.server.xml")+1);
		sprintf(ret, "%s/.sussen.server.xml", home);
		return(ret);
	}

	pwd = getpwuid(getuid());
	if (pwd && pwd->pw_dir) {
		ret = g_malloc(strlen(pwd->pw_dir)+strlen("/.sussenrc")+1);
		sprintf(ret, "%s/.sussen.server.xml", pwd->pw_dir);
		return(ret);
	} 
	
	return(NULL);
}

gboolean check_for_server_xml (void) {
	
	GIOChannel *channel;
	GError *err = NULL;
	const gchar *filename;
	
	filename = get_server_xml_filename ();
	channel = g_io_channel_new_file (filename, "r", &err);
	
	if (err) {
		g_free (err);
		return FALSE;
	} 
	
	g_io_channel_shutdown (channel, TRUE, NULL);
	return TRUE;
}

void set_server_xml (GSList *list) {
	
	gchar *buf = g_malloc (2000);
	const gchar *filename;
	gsize *bytes_written = 0;
	gint status;
	

	filename = get_server_xml_filename ();
	channel = g_io_channel_new_file (filename,"w+", NULL);
	
	if (channel) {

		g_snprintf (buf,2000,"<servers>\n");
		status = g_io_channel_write_chars (channel,buf,-1,bytes_written,NULL);

		g_slist_foreach  (list, list_callback, NULL);
		
		g_snprintf (buf,2000,"</servers>\n");
		status = g_io_channel_write_chars (channel,buf,-1,bytes_written,NULL);
		g_io_channel_shutdown (channel, TRUE, NULL);
	}	
	
	g_free (buf);
}

GSList* get_server_xml (void) {
	
	GSList *server_list = NULL;
	xmlDocPtr doc;
	xmlNodePtr cur;
	const gchar *filename;
	
	filename = get_server_xml_filename ();
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                      GTK_DIALOG_DESTROY_WITH_PARENT,
                   					  GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
								"could not find server.xml");
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		return NULL;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {

		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                      GTK_DIALOG_DESTROY_WITH_PARENT,
                   					  GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
								"corrupt or invalid server.xml");
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		

		xmlFreeDoc(doc);
		return NULL;		
    }	
		
	cur = cur->xmlChildrenNode;
	
	while (cur != NULL) {

		gchar *name;
		gchar *port;
		gchar *user;
		gchar *enabled;
		gchar *list_str;
		
		list_str = g_malloc (255);
		
		name = xmlGetProp(cur, (const xmlChar *) "name");
		port = xmlGetProp(cur, (const xmlChar *) "port");
		user = xmlGetProp(cur, (const xmlChar *) "user");
		enabled = xmlGetProp(cur, (const xmlChar *) "enabled");
		
		g_snprintf (list_str, 255, "%s:%s:%s:%s", name, port, user, enabled);
		server_list = g_slist_append (server_list, g_strdup(list_str));
		
		g_free (list_str);	
		cur = cur->next;
	}
	
	return server_list;	
}
