/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - policy-callback.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-util.h"
#include "sussen-policy-callback.h"

#include <includes.h>
#include "nessus.h"
#include "globals.h"
#include "parser.h"
#include "families.h"

/* ADAM */
#include <glib.h>
#include "comm.h"
#include "preferences.h"

GtkWidget *file_dialog;

GtkWidget *port_scanner_tree;
GtkListStore *port_scanner_store;

GtkWidget *port_scanner_prefs_tree;
GtkListStore *port_scanner_prefs_store;

GtkTreeIter family_iter;
GtkTreeIter plugin_iter;
GtkTreeIter scanner_iter;

typedef struct plugin {
    xmlChar *plugID;
    xmlChar *enabled;
} plugin, *pluginPtr;

void set_policy_file (GtkWidget *widget, gpointer user_data) {

	GIOChannel *policy_channel;
	gchar *buf = g_malloc (2000);
	const gchar *filename;
	gsize *bytes_written = 0;
	gint status;
	

	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	policy_channel = g_io_channel_new_file (filename,"w+", NULL);
	
	if (policy_channel) {

		g_snprintf (buf,sizeof(buf),"<plugins>\n");
		status = g_io_channel_write_chars (policy_channel,buf,-1,bytes_written,NULL);
		
		struct arglist *plugs = Plugins;
		while (plugs && plugs->next) {

/*
		FIXME: The plugin prefs are not XML escaped.  g_markup_escape_text()
			   butchers the values pretty bad, need to find another way to
			   deal with that.  
			
			struct arglist *prefs;
			if (prefs = arg_get_value (plugs->value,"plugin_prefs")) {
			
				g_sprintf (buf,"<plugin id=\"%s\" enabled=\"%d\">\n",
						plugin_asc_id(plugs->value),(int)plug_get_launch(plugs->value));
				status = g_io_channel_write_chars (policy_channel,buf,-1,&bytes_written,NULL);
			
				while(prefs && prefs->next) {
					
					gchar *value;
					gchar *xml_clean_value;
					
					value = arg_get_value(prefs->value, "value");
					xml_clean_value = g_markup_escape_text (value, sizeof(arg_get_value(prefs->value, "value")));
					
					g_print ("name == [%s] -- value == [%s] -- clean == [%s]\n", prefs->name, value, xml_clean_value);
					g_sprintf (buf,"<pref name=\"%s\" value=\"%s\" />\n", prefs->name, xml_clean_value);
					g_io_channel_write_chars (policy_channel,buf,-1,&bytes_written,NULL);
					
					prefs = prefs->next;
				}

				g_sprintf (buf,"</plugin>\n");							
			} else {
*/			
			g_snprintf (buf,2000,"<plugin id=\"%s\" enabled=\"%d\" />\n",plugin_asc_id(plugs->value),(int)plug_get_launch(plugs->value));
/*				
			}
*/			
			status = g_io_channel_write_chars (policy_channel,buf,-1,bytes_written,NULL);
			
			if (status != G_IO_STATUS_NORMAL) {

				switch (status) {
	
					case G_IO_STATUS_ERROR:
						g_print ("G_IO_STATUS_ERROR\n");
						break;
				
					case G_IO_STATUS_EOF:
						g_print ("G_IO_STATUS_EOF\n");
						break;
			
					case G_IO_STATUS_AGAIN:
						g_print ("G_IO_STATUS_AGAIN\n");
						break;
				}	
			}
		
		
			plugs = plugs->next;
		}

		g_snprintf (buf,2000,"</plugins>\n");
		status = g_io_channel_write_chars (policy_channel,buf,-1,bytes_written,NULL);
		
		g_io_channel_shutdown (policy_channel, TRUE, NULL);
	}	
	
	g_free (buf);

}

void on_quit_policy_manager_activate (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;

	w = glade_xml_get_widget (policy_manager, "policy_manager_window");	
	gtk_widget_destroy (w);
}

void on_enable_dependencies_activate (GtkWidget *widget, gpointer user_data) {
	
	gchar *s;
	
	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(widget))) {
		s = g_strdup("yes");	
	} else {
		s = g_strdup("no");
	}
	
	arg_set_value(Prefs, "auto_enable_dependencies", strlen(s), s);	
}

gboolean load_policy_xml (const gchar *filename) {

	xmlDocPtr doc;
	xmlNodePtr cur;
	
	doc = xmlParseFile(filename);
	if (doc == NULL) {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                      GTK_DIALOG_DESTROY_WITH_PARENT,
                   					  GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
								"Invalid policy filename");
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		return FALSE;
	}
	
    cur = xmlDocGetRootElement(doc);
    if (cur == NULL) {

		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                      GTK_DIALOG_DESTROY_WITH_PARENT,
                   					  GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
								"Policy file corrupt or invalid");
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		

		xmlFreeDoc(doc);
		return FALSE;		
    }	
	
	pluginPtr ret = NULL;	
	ret = (pluginPtr) g_malloc (sizeof(plugin));
	if (ret == NULL ) {
		update_status ("Trying to load policy file: out of memory");
	}
	bzero (ret, sizeof(plugin));
	
	cur = cur->xmlChildrenNode;
	
	while (cur != NULL) {

		char *id;
		char *enabled;
		
		id = xmlGetProp(cur, (const xmlChar *) "id");
		enabled = xmlGetProp(cur, (const xmlChar *) "enabled");
		
		g_hash_table_insert (plugin_hash,id,enabled);
		cur = cur->next;
	}
	
	use_policy_file = 1;
	return TRUE;
}

void load_policy (GtkWidget *widget, gpointer user_data) {
	
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	load_policy_xml (filename);
}

void on_policy_manager_help_activate (GtkWidget *widget, gpointer user_data) {

	GError *err = NULL;

	if (!gnome_help_display ("sussen.xml","sussen-policy-manager",&err)) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
	
}

void on_policy_open_activate (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Load Policy");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (load_policy),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
	
	
}

void on_policy_manager_save_activate (GtkWidget *widget, gpointer user_data) {

	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Save Policy");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_policy_file),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void upload_plugin (GtkWidget *widget, gpointer user_data) {

	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	comm_plugin_upload (filename);
}

void on_upload_plugin_activate (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select Plugin to Upload");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (upload_plugin),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_port_scanner_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                      gchar *path,
        		    		  gpointer user_data) {
	gboolean enabled;	
	GtkTreeIter iter;
	
	gchar *cur_scanner;
	struct arglist *scan = Scanners;

	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (port_scanner_store), &iter, path);								  
	gtk_tree_model_get (GTK_TREE_MODEL (port_scanner_store), &iter, 
						0, &enabled,
						1, &cur_scanner,
						-1);
	
	while (scan && scan->next) {
	
		if (strcmp (cur_scanner,scan->name) == 0) {
			plug_set_launch (scan->value,!enabled);
		}
	
		scan = scan->next;
	}

	pluginset_reload(Plugins, Scanners);
	gtk_list_store_set (GTK_LIST_STORE (port_scanner_store), &iter, 0, !enabled, -1);
	
	g_free (cur_scanner);
}

void on_port_scanner_clicked (GtkTreeSelection *selection, gpointer data) {

	struct arglist *scans = Scanners;
	struct arglist *plugin;
	gchar *cur_scanner;
	
	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(port_scanner_tree));
	gtk_tree_selection_get_selected (sel,&global_model,&scanner_iter);

	gtk_tree_model_get (GTK_TREE_MODEL(port_scanner_store),&scanner_iter,
							1,&cur_scanner,
							-1);

	if (cur_scanner != NULL) {
		plugin = arg_get_value (scans, cur_scanner);
	
		struct arglist * prefs;
		if ((prefs = arg_get_value (plugin, "plugin_prefs"))) {
			gtk_list_store_clear (port_scanner_prefs_store);
		
			while(prefs && prefs->next) {
				
				gchar * type, *value;
				
				type  = arg_get_value(prefs->value, "type");
				value = arg_get_value(prefs->value, "value");
				
				if(type) {
				
					GtkTreeIter iter;

					gtk_list_store_append (port_scanner_prefs_store, &iter);
					gtk_list_store_set (port_scanner_prefs_store, &iter,
										0, prefs->name,  
										1, value,
										-1);										
				}		
				
				prefs = prefs->next;
			}				
		}
			
		g_free (cur_scanner);
	}
}

void on_port_scan_settings_activate (GtkWidget *widget, gpointer user_data) {

	GladeXML *xml;
	GtkWidget *dialog;
	GtkWidget *w;
		
    xml = glade_xml_new (PACKAGE_DATA_DIR"/port_scanner_settings.glade", NULL, NULL);
	dialog = glade_xml_get_widget (xml, "port_scanner_settings_dialog");
		
	/* destroy dialog on button press */
	w = glade_xml_get_widget (xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked", 
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer)dialog);
	

	/*
	 * Configure port_scanner_listview
	 */
	port_scanner_tree = glade_xml_get_widget (xml,"port_scanner_listview");
	port_scanner_store = gtk_list_store_new (2,G_TYPE_BOOLEAN,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (port_scanner_tree), GTK_TREE_MODEL (port_scanner_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer, *toggle_renderer;

		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_port_scanner_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_tree), column);		
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Scanner",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_tree), column);	
	}	

	/*
	 * Configure port_scanner_prefs_listview
	 */
	port_scanner_prefs_tree = glade_xml_get_widget (xml,"port_scanner_prefs_listview");
	port_scanner_prefs_store = gtk_list_store_new (2,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (port_scanner_prefs_tree), GTK_TREE_MODEL (port_scanner_prefs_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Name",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_prefs_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Value",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (port_scanner_prefs_tree), column);	
		
	}	
		
	GtkTreeSelection *port_scanner_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(port_scanner_tree));
	gtk_tree_selection_set_mode (port_scanner_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(port_scanner_selection), "changed",  G_CALLBACK(on_port_scanner_clicked), NULL);
	
	struct arglist *scans = Scanners;
	GtkTreeIter iter;
	
	while(scans && scans->next) {
			
		gtk_list_store_append (GTK_LIST_STORE(port_scanner_store), &iter);
		gtk_list_store_set (port_scanner_store, &iter,
							0, (int)plug_get_launch(scans->value), 
							1, scans->name,							
							-1);
		scans = scans->next;
	}
	
}

gboolean enable_all_but_dos_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;

	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
		family_enable(cur_family, Plugins, ENABLE_FAMILY_BUT_DOS);
		if ( family_enabled(cur_family, Plugins) ) { 
			gtk_list_store_set (GTK_LIST_STORE (model), iter, 
								0, TRUE, 
								-1);
		} else {
			gtk_list_store_set (GTK_LIST_STORE (model), iter, 
								0, FALSE, 
								-1);			
		}
			g_free (cur_family);
			return FALSE;
	}
	return TRUE;
}

void on_enable_all_but_dangerous_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), enable_all_but_dos_each, NULL);
	pluginset_reload(Plugins, Scanners);	
}

gboolean enable_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;
	
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
		family_enable(cur_family, Plugins, ENABLE_FAMILY);
		gtk_list_store_set (GTK_LIST_STORE (model), iter, 
							0, TRUE, 
							-1);
		g_free (cur_family);
		return FALSE;
	}
	return TRUE;
}

void on_enable_all_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), enable_each, NULL);
	pluginset_reload(Plugins, Scanners);
	
}

gboolean disable_each (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	gchar *cur_family;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(model),iter,1,&cur_family,-1);
		family_enable(cur_family, Plugins, DISABLE_FAMILY);
		gtk_list_store_set (GTK_LIST_STORE (model), iter, 
							0, FALSE, 
							-1);
		g_free (cur_family);
		return FALSE;
	}
	
	return TRUE;
}

void on_disable_all_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(plug_family_store), disable_each, NULL);
	pluginset_reload(Plugins, Scanners);
	
}

void on_plugin_clicked (GtkTreeSelection *selection, gpointer data) {

	struct arglist * plugs = Plugins;
	struct arglist * plugin;
	gchar *cur_plugin;
	GtkWidget *w;
	GladeXML *xml;
	
	xml = policy_manager;
	
	GtkTreeSelection *sel = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_tree));
	gtk_tree_selection_get_selected (sel,&global_model,&plugin_iter);

	gtk_tree_model_get (GTK_TREE_MODEL(plug_store),&plugin_iter,
							0,&cur_plugin,
							-1);

	if (cur_plugin != NULL) {
		plugin = arg_get_value (plugs, cur_plugin);
	

		if(plugin) {
			if(arg_get_value(plugin, "DESCRIPTION")) {
				GtkTextBuffer *buffer;
				gchar *text_to_insert;
			
				text_to_insert = arg_get_value(plugin, "DESCRIPTION");
				
				buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(plugin_description_textview));
				gtk_text_buffer_set_text (buffer,text_to_insert,-1);
				gtk_text_view_set_buffer (GTK_TEXT_VIEW(plugin_description_textview),buffer);
			}	

			struct arglist * prefs;
			if ((prefs = arg_get_value (plugin, "plugin_prefs"))) {
				gtk_list_store_clear (plug_pref_store);
				w = glade_xml_get_widget (xml,"plugin_pref_name_label");
				gtk_label_set_text (GTK_LABEL(w), cur_plugin);
			
				while(prefs && prefs->next) {
					
					gchar * type, *value;
					
					type  = arg_get_value(prefs->value, "type");
					value = arg_get_value(prefs->value, "value");
					
					if(type) {
						
						GtkTreeIter iter;
		
						gtk_list_store_append (plug_pref_store, &iter);
						gtk_list_store_set (plug_pref_store, &iter,
											0, prefs->name,  
											1, value,
											-1);										
					}		
					
					prefs = prefs->next;
				}				
			}
			
			g_free (cur_plugin);
		}
	}
}

void on_plugin_family_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                                gchar *path,
        		                        gpointer user_data) {
	gboolean enabled;
	gchar *cur_family;							
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plug_family_store), &iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (plug_family_store), &iter, 
						0, &enabled, 							
						1, &cur_family,
						-1);
	gtk_list_store_set (GTK_LIST_STORE (plug_family_store), &iter, 0, !enabled, -1);
	family_enable (cur_family,Plugins,enabled);	
	g_free (cur_family);

}

void on_plugin_toggled (GtkCellRendererToggle *cellrenderertoggle,
		                                gchar *path,
        		                        gpointer user_data) {
	gchar *cur_plugin;
	gboolean enabled;
	GtkTreeIter iter;
	struct arglist *plug = Plugins;
	
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (plug_store), &iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (plug_store), &iter, 
						0, &cur_plugin,
						2, &enabled, 
						-1);
	gtk_list_store_set (plug_store, &iter, 2, !enabled, -1);

	while (plug && plug->next) {
		
		if (strcmp (cur_plugin,plug->name) == 0) {
			plug_set_launch (plug->value,enabled);
		}
		
		plug = plug->next;
	}

	
	g_free (cur_plugin);
}


void on_plugin_family_clicked (GtkTreeSelection *selection, gpointer data) {
	
	struct arglist * plugs = Plugins;
	gchar *cur_family;
	GdkPixbuf *warn;
	
	gtk_tree_selection_get_selected (selection,&global_model,&family_iter);
	gtk_tree_model_get (GTK_TREE_MODEL(plug_family_store),&family_iter,
							1,&cur_family,
							-1);
	
	if(plugs)
	{
		gtk_list_store_clear (plug_store);
		
		while(plugs->next) {
	   
			if(arg_get_value(plugs->value, "FAMILY") 
				&& !strcmp(arg_get_value(plugs->value, "FAMILY"), cur_family))
			{
				char *cat = arg_get_value(plugs->value, "CATEGORY");
				int warning = cat? (!strcmp(cat, "denial") ||
									!strcmp(cat, "kill_host") ||
									!strcmp(cat, "destructive_attack")):0;



				
				if (warning) {
					warn = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/warning.png",NULL);
				}
				else {
					warn = gdk_pixbuf_new_from_file ("nothing.xpm",NULL);
				}
				
				GtkTreeIter iter;
		
				gtk_list_store_append (plug_store, &iter);
				gtk_list_store_set (plug_store, &iter,
									0, plugs->name,  
									1,warn,
									2,(int)plug_get_launch(plugs->value),							
									-1);				
			}

		plugs = plugs->next;
  		}
 	}

	g_free ( cur_family );	
}

void show_policy_manager (void) {

	GtkWidget *policy_window;
	static struct plugin_families *families;
		
	/* 
	 * Create Policy Manager window
	 */
	policy_manager = glade_xml_new (PACKAGE_DATA_DIR"/policy_manager.glade", NULL, NULL);
	policy_window = glade_xml_get_widget (policy_manager, "policy_manager_window");
	
	/* auto-connect signals */
	glade_xml_signal_autoconnect (policy_manager);
	
	/*
	 * Get rid of window after user closes it
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (policy_window),
							  "destroy",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) policy_window);
	

	/* 
	 * setup listviews
	 */
	  
	GtkTreeIter iter;

	plug_family_tree = glade_xml_get_widget (policy_manager,"plugin_family_listview");
	plug_family_store = gtk_list_store_new (2, G_TYPE_BOOLEAN, G_TYPE_STRING);	
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_family_tree), GTK_TREE_MODEL (plug_family_store));
   {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer, *toggle_renderer;

		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_plugin_family_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_family_tree), column);		

		
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Plugin Family Name",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);   
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_family_tree), column);			
	}

	struct arglist *plugs = Plugins;
	struct plugin_families *lfamilies;
			
	lfamilies = families = family_init();
	if(plugs)
	while(plugs->next) {
		family_add(families, plugs->value);
		plugs = plugs->next;
	}
	plugs = Plugins;		

	if(lfamilies)
	while(lfamilies->next) {
			
		struct plugin_families * old = arg_get_value(MainDialog, "FAMILIES");
		int flag = 0;
		if(old)
			while(old->next && !flag) {
				if(old->name)flag = !strcmp(old->name, lfamilies->name);
					old = old->next;
			}
     
		if(flag){
			lfamilies = lfamilies->next;
			continue;
		}

	gtk_list_store_append (GTK_LIST_STORE(plug_family_store), &iter);
	gtk_list_store_set (plug_family_store, &iter,
						0, (family_enabled(lfamilies->name, Plugins) ? TRUE:FALSE),
						1, lfamilies->name,  						
						-1);	
			
	lfamilies = lfamilies->next;			
	}

 	plug_tree = glade_xml_get_widget (policy_manager,"plugins_listview");
	plug_store = gtk_list_store_new (3, G_TYPE_STRING, GDK_TYPE_PIXBUF, G_TYPE_BOOLEAN);
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_tree), GTK_TREE_MODEL (plug_store));
    {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer, *pixbuf_renderer, *toggle_renderer;
		
		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_plugin_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
														"active",2,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 2);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		

		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Dangerous",pixbuf_renderer,
															"pixbuf",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		
		
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Plugin Name",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_tree), column);		
	}

 	plug_pref_tree = glade_xml_get_widget (policy_manager,"prefs_listview");
	plug_pref_store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (plug_pref_tree), GTK_TREE_MODEL (plug_pref_store));
    {
	    GtkTreeViewColumn *column;
	    GtkCellRenderer *text_renderer;
			
		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Name",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);		
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_pref_tree), column);	


		text_renderer = gtk_cell_renderer_text_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Value",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);				
		gtk_tree_view_append_column (GTK_TREE_VIEW (plug_pref_tree), column);				
	}
	
	
	GtkTreeSelection *family_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_family_tree));		
	gtk_tree_selection_set_mode (family_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(family_selection), "changed",  G_CALLBACK(on_plugin_family_clicked), NULL);

	GtkTreeSelection *plugin_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(plug_tree));
	gtk_tree_selection_set_mode (plugin_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(plugin_selection), "changed",  G_CALLBACK(on_plugin_clicked), NULL);

	plugin_description_textview = glade_xml_get_widget (policy_manager,"plugin_description_textview");
}
