/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-preferences.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-fileio.h"
#include "sussen-gconf.h"

GladeXML *xml;
GladeXML *server_xml;
GtkWidget *main_dialog = NULL;
GtkWidget *server_tree;
GtkListStore *server_store;
GtkTreeSelection *selected;
GtkTreeIter selected_iter;
gint add = 0;
GSList *new_list = NULL;

void on_server_ok_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *w;
	
	const gchar *host;
	const gchar *port;
	const gchar *user;
	
	w = glade_xml_get_widget (server_xml, "host_entry");
	host = gtk_entry_get_text (GTK_ENTRY(w));
	
	w = glade_xml_get_widget (server_xml, "port_entry");
	port = gtk_entry_get_text (GTK_ENTRY(w));

	w = glade_xml_get_widget (server_xml, "user_entry");
	user = gtk_entry_get_text (GTK_ENTRY(w));
	
	if (add) {
		GtkTreeIter iter;

		gtk_list_store_append (GTK_LIST_STORE(server_store),&iter);
		gtk_list_store_set (GTK_LIST_STORE (server_store), &iter, 
							0, 1,
							1, g_strdup(host), 
							2, g_strdup(port),
							3, g_strdup(user),
							-1);	
		add = 0;
	} else {
		if (gtk_list_store_iter_is_valid (server_store,&selected_iter)) {
			gtk_list_store_set (GTK_LIST_STORE (server_store), &selected_iter, 
								1, g_strdup(host), 
								2, g_strdup(port),
								3, g_strdup(user),
								-1);
		}
	}
}

void show_server_dialog (gchar *title, gchar *host, gchar *port, gchar *user) {
	
	GtkWidget *dialog;
	GtkWidget *w;

    server_xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen_preferences_server.glade", NULL, NULL);
	dialog = glade_xml_get_widget (server_xml, "server_dialog");
		
	w = glade_xml_get_widget (server_xml, "ok_button");
	g_signal_connect (G_OBJECT(w),
					  "clicked",
					  G_CALLBACK (on_server_ok_clicked),
					  NULL);

	/* Get rid of dialog after user clicks a button */		
	w = glade_xml_get_widget (server_xml, "cancel_button");
	g_signal_connect_swapped (G_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) dialog);
	
	w = glade_xml_get_widget (server_xml, "ok_button");
	g_signal_connect_swapped (G_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) dialog);
	
	/* populate dialog */
	gtk_window_set_title (GTK_WINDOW(dialog),g_strdup(title));
	
	w = glade_xml_get_widget (server_xml, "host_entry");
	gtk_entry_set_text (GTK_ENTRY(w), g_strdup(host));
	
	w = glade_xml_get_widget (server_xml, "port_entry");
	gtk_entry_set_text (GTK_ENTRY(w), g_strdup(port));

	w = glade_xml_get_widget (server_xml, "user_entry");
	gtk_entry_set_text (GTK_ENTRY(w), g_strdup(user));	
}

void on_add_server_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	add++;
	show_server_dialog (_("Add Server"),"","","");		
}

void on_edit_server_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(server_tree));
	gtk_tree_selection_get_selected (selected,NULL,&selected_iter);
	if (gtk_list_store_iter_is_valid (server_store,&selected_iter)) {
		
		gchar *host;
		gchar *port;
		gchar *user;
		
		gtk_tree_model_get (GTK_TREE_MODEL(server_store),&selected_iter,
							1,&host,
							2,&port,
							3,&user,
							-1);
		
		show_server_dialog (_("Edit Server"),host,port,user);
		
		g_free (user);
		g_free (port);
		g_free (host);
		
	} else {
		GtkWidget *dialog = NULL;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_dialog),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              "You must select a server to edit");
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);				
	}	
		
}

void on_remove_server_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkTreeSelection *selected;
	GtkTreeIter iter;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(server_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	if (gtk_list_store_iter_is_valid (server_store,&iter)) {
		gtk_list_store_remove (server_store,&iter);	
	} else {
		GtkWidget *dialog = NULL;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_dialog),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              "You must select a server to delete");
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);				
	}	
}

void on_server_toggled (GtkCellRendererToggle *cellrenderertoggle, gchar *path, gpointer user_data) {
											
	gboolean enabled;
	GtkTreeIter iter;
	
	gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (server_store), &iter, path);
	gtk_tree_model_get (GTK_TREE_MODEL (server_store), &iter, 
						0, &enabled, 							
						-1);
	gtk_list_store_set (GTK_LIST_STORE (server_store), &iter, 0, !enabled, -1);
												
}

void server_list_callback (gpointer data, gpointer user_data) {
	
	GtkTreeIter iter;
	gchar *host;
	gchar *port;
	gchar *user;
	gchar *enabled;
	
	gchar *cp;
	gchar *tmp;

	cp = g_strdup ((gchar*)data);
	tmp = g_malloc (255);	
		
	host = strtok (cp, ":");  
	port = strtok (NULL, ":");
	user = strtok (NULL, ":");
	enabled = strtok (NULL, ":");
	
	gtk_list_store_append (GTK_LIST_STORE(server_store), &iter);
	gtk_list_store_set (server_store, &iter,
						0, atoi(enabled),
						1, g_strdup (host),
						2, g_strdup (port),
						3, g_strdup (user),
						-1);	
	
	g_free (tmp);
}

void for_each_server (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data) {
	
	gchar *tmp;
	gchar *host;
	gchar *port;
	gchar *user;
	gboolean enabled;

	gtk_tree_model_get (GTK_TREE_MODEL(model),iter,
						0,&enabled,
						1,&host,
						2,&port,
						3,&user,
						-1);

	
	tmp = g_malloc (255);
	snprintf (tmp, 255, "%s:%s:%s:%d", host, port, user, enabled);
	new_list = g_slist_append (new_list, g_strdup(tmp));
	
	g_free (tmp);
	g_free (user);
	g_free (port);
	g_free (host);
}

void on_preferences_ok_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	gtk_tree_model_foreach (GTK_TREE_MODEL(server_store),(void *)for_each_server, NULL);
	if (new_list) {
		set_server_xml (g_slist_copy(new_list));
	}		
}

void show_preferences_dialog (void) {

	GtkTreeSelection *selected;
	GtkWidget *w;

    xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen_preferences.glade", NULL, NULL);
	main_dialog = glade_xml_get_widget (xml, "sussen_preferences_dialog");
		
    glade_xml_signal_autoconnect(xml); 	

	w = glade_xml_get_widget (xml, "ok_button");
	g_signal_connect (G_OBJECT(w),
					  "clicked",
					  G_CALLBACK (on_preferences_ok_button_clicked),
					  NULL);


	/* Get rid of dialog after user clicks a button */		
	w = glade_xml_get_widget (xml, "close_button");
	g_signal_connect_swapped (G_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) main_dialog);
	
	w = glade_xml_get_widget (xml, "ok_button");
	g_signal_connect_swapped (G_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) main_dialog);
							  
	server_tree = glade_xml_get_widget (xml,"servers_listview");
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(server_tree));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION(selected), GTK_SELECTION_SINGLE);
	server_store = gtk_list_store_new (4,G_TYPE_BOOLEAN,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (server_tree), GTK_TREE_MODEL (server_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *toggle_renderer, *text_renderer;

		toggle_renderer = gtk_cell_renderer_toggle_new ();
		g_signal_connect (toggle_renderer, "toggled", G_CALLBACK (on_server_toggled), NULL);
		column = gtk_tree_view_column_new_with_attributes ("Enabled",toggle_renderer,
															"active",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_tree), column);	
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Server",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_tree), column);	

		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Port",text_renderer,
															"text",2,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 2);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("User",text_renderer,
															"text",3,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 3);
		gtk_tree_view_append_column (GTK_TREE_VIEW (server_tree), column);		
	}		

	GSList *the_list = NULL;
	
	the_list = get_server_xml ();
	g_slist_foreach  (the_list, server_list_callback, NULL);
}
