/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-util.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-util.h"

#include <includes.h>
#include "nessus.h"
#include "globals.h"

GtkWidget* lookup_widget (GtkWidget *widget, const gchar *widget_name) {

	GtkWidget *parent, *found_widget;

	for (;;) {

		if (GTK_IS_MENU (widget))
			parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
		else
			parent = widget->parent;

		if (!parent)
			parent = (GtkWidget*) g_object_get_data (G_OBJECT (widget), "GladeParentKey");
		if (parent == NULL)
			break;
		
		widget = parent;
	}

	found_widget = (GtkWidget*) g_object_get_data (G_OBJECT (widget),
	widget_name);
	
	if (!found_widget)
		g_warning ("Widget not found: %s", widget_name);
	
	return found_widget;
}

void update_status (gchar *msg) {
	
	GtkWidget *appbar;
	
	appbar = glade_xml_get_widget (mainwindow_xml,"sussen_app_bar");
	gnome_appbar_set_status (GNOME_APPBAR(appbar),g_strdup(msg));
}

void set_widgets_sensitive (gboolean state, gboolean during_scan) {
	
	GtkWidget *w;

	w = glade_xml_get_widget (mainwindow_xml, "targets_entry");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "dns_transfer_checkbutton");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "save_session_checkbutton");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "target_browse_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_use_current_radio");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "policy_entry");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "policy_browse_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "policy_manager_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "start_scan_button");
	gtk_widget_set_sensitive (w,state);
	
	w = glade_xml_get_widget (mainwindow_xml, "start_scan1");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "properties_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "properties1");
	gtk_widget_set_sensitive (w,state);
		
	w = glade_xml_get_widget (mainwindow_xml, "session_button");
	gtk_widget_set_sensitive (w,state);

	w = glade_xml_get_widget (mainwindow_xml, "sessions1");
	gtk_widget_set_sensitive (w,state);

	if (!during_scan) {
	
		w = glade_xml_get_widget (mainwindow_xml, "disconnect_button");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "disconnect1");
		gtk_widget_set_sensitive (w,state);
				
		w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "scan_status_listview");
		gtk_widget_set_sensitive (w,state);

		w = glade_xml_get_widget (mainwindow_xml, "stop_scanning_host_button");
		gtk_widget_set_sensitive (w,state);	
	}
}

void set_target_defaults (void) {

#define EX_NONE 1
#define EX_DNS 2
#define EX_IP 4
#define EX_NFS 8
	
	struct arglist *serv;
	char *v;
	int flag = 0;
	GtkWidget *widget;

	/* widgets on 'Targets' tab */
	
	serv = arg_get_value(Prefs, "SERVER_PREFS");
	v = arg_get_value(serv, "host_expansion");
	
	if(!v)
		flag |= EX_NONE;
	else {
		if(strstr(v, "dns"))flag = EX_DNS;
		if(strstr(v, "nfs"))flag |= EX_NFS;
		if(strstr(v, "none"))flag |= EX_NONE;
		if(strstr(v, "ip"))flag |= EX_IP;
	}
 
	if(!(flag & EX_NONE)) {
		widget = glade_xml_get_widget (mainwindow_xml,"dns_transfer_checkbutton");	
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (flag & EX_DNS));
	}	

	v = arg_get_value(serv, "save_session");
	widget = glade_xml_get_widget (mainwindow_xml,"save_session_checkbutton");
	if (arg_get_type(serv, "save_session") == ARG_INT) {
		char * s = strdup(v ? "yes":"no");
		arg_set_type(serv, "save_session", ARG_STRING);
		arg_set_value(serv, "save_session", strlen(s), s);
		v = s;
	}
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), (v && !strcmp(v, "yes")));	
	
}
